#!/usr/bin/env python
# coding=utf-8
'''
brief        :
Author       : knightdby knightdby@163.com
Date         : 2023-02-24 23:40:06
FilePath     : /manifast/manifast/label_utils/segment/gen_split_data.py
Description  :
LastEditTime : 2023-02-25 22:17:04
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''

from manifast import *

from convert_utils import cropRotateBox, rotateImagebyHeightLonger


def genGTDatabase(image_path, idlabel_path, label2num_dict={}, save_crop_dir='./tmp/database/seg'):
    '''
    description: 生成ground truth database
    param       {*} image_path 图像路径
    param       {*} txt_path 标注路径
    param       {*} save_crop_dir 数据库保存路径
    return      {*} crop image path list
    '''
    image = readImageFilebyCv(image_path)
    img_name = os.path.basename(image_path)[:-4]
    label_img = readImageFilebyCv(idlabel_path)
    if not label2num_dict:
        for i in range(np.max(label_img)+1):
            label2num_dict[str(i)] = i
    for item in label2num_dict.items():
        mask = np.where(label_img == item[1], 1, 0)
        mask = np.array(mask, np.uint8)
        class_img = np.multiply(image, mask[:, :, np.newaxis])
        im, contours, hierarchy = cv2.findContours(
            mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
        cnts = {}
        for cidx, cnt in enumerate(contours):
            if cv2.contourArea(cnt) < 200:
                continue
            minAreaRect = cv2.minAreaRect(cnt)
            rectCnt = np.int64(cv2.boxPoints(minAreaRect))
            cnts[cidx] = (rectCnt[1][0]+rectCnt[3][0])/2
            cv2.polylines(img=class_img, pts=[rectCnt], isClosed=True, color=(
                0, 0, 255), thickness=3)
        sort_cnts = sorted(cnts.items(), key=lambda x: x[1])  # 对字典进行排序
        for idx, key in enumerate(sort_cnts):
            # if item[1]==6:
            #     if cv2.contourArea(contours[key[0]]) < 500:
            #         save_name='sml-'+img_name+f'{-cv2.contourArea(contours[key[0]])}'
            #     else:
            #         save_name=img_name
            # elif item[1]==14:
            #     if cv2.contourArea(contours[key[0]]) < 1500:
            #         save_name='sml-'+img_name+f'{-cv2.contourArea(contours[key[0]])}'
            #     else:
            #         save_name=img_name
            # else:
            #     save_name=img_name
            save_name = img_name
            minAreaRect = cv2.minAreaRect(contours[key[0]])
            rectCnt = np.int64(cv2.boxPoints(minAreaRect))
            wrap_img = cropRotateBox(contours[key[0]], class_img)
            rotate_img = rotateImagebyHeightLonger(wrap_img)
            save_path = os.path.join(save_crop_dir, item[0]+'/'+save_name+'_'+str(idx) +
                                     '.jpg')
            makePathDirs(save_path)
            cv2.imwrite(save_path, rotate_img[:, :, ::-1])


def genSplitData(dataset_dir, label_paths, dataset_name, model='train'):
    save_path = dataset_dir + \
        f"/data_info/{dataset_name}_{model}_list.txt"
    makePathDirs(save_path)
    with open(save_path, 'w') as f:
        pass
    for label_path in label_paths:
        label_name = os.path.basename(label_path)
        with open(save_path, 'a') as f:
            if os.path.exists(dataset_dir+'/images/'+label_name.replace('.png', '.jpg')):
                f.write('images/'+label_name.replace('.png', '.jpg')+' '
                        + 'labelIds/'+label_name+'\n')
            elif os.path.exists(dataset_dir+'/images/'+label_name):
                f.write('images/'+label_name+' '
                        + 'labelIds/'+label_name+'\n')
            else:
                pass


if __name__ == "__main__":
    image_path = '/home/knight/waytous/dataset/minedatset/images/sence_0_000106.jpg'
    idlabel_path = image_path.replace(
        'images', 'labelIds').replace('.jpg', '.png')
    genGTDatabase(image_path, idlabel_path)



#!/usr/bin/env python
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-02-24 20:55:47
FilePath     : /wheel/manifast/model_analysis/summary.py
Description  : 
LastEditTime : 2023-02-24 21:01:54
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''


import numpy as np
from tqdm import tqdm
import time
import torch
# from thop import profile
# from thop import clever_format


def model_visual(model):
    blank = ' '
    print('-----------------------------------------------')
    print('|   weight name   |        weight shape       |')
    print('-----------------------------------------------')

    for index, (key, w_variable) in enumerate(model.named_parameters()):
        if len(key) <= 60:
            key = key + (60 - len(key)) * blank
        w_variable_blank = ''
        if len(w_variable.shape) == 1:
            if w_variable.shape[0] >= 120:
                w_variable_blank = 8 * blank
            else:
                w_variable_blank = 9 * blank
        elif len(w_variable.shape) == 2:
            if w_variable.shape[0] >= 120:
                w_variable_blank = 2 * blank
            else:
                w_variable_blank = 3 * blank

        print('| {} | {}{} |'.format(key, w_variable.shape, w_variable_blank))
        key = 0
    print('-----------------------------------------------')


# def model_summary(model, inputs=[torch.randn(1, 3, 640, 480), torch.randn(1, 1, 640, 480)]):
#     if len(inputs) == 2:
#         flops, params = profile(model, inputs=(inputs[0], inputs[1]))
#     else:
#         flops, params = profile(model, inputs=(inputs[0],))
#     flops, params = clever_format([flops, params], "%.3f")

#     type_size = 1
#     print('-' * 90)
#     print('#Model: {} | Params: {} | FLOPs: {}'.format(
#         model._get_name(), params, flops))
    # print('-' * 90)


def summary(model, input):
    model_visual(model)
    # pytorch_total_params = sum(p.numel() for p in net.parameters())
    # print("Total number of parameters: %.3fM" % (pytorch_total_params / 1e6))
    model = model.cuda()
    model.eval()
    run_time = list()
    if type(input) is list or type(input) is tuple:
        input = [data.cuda() for data in input]
    else:
        input = input.cuda()
    # input = torch.randn(2, 3, height, width).cuda()
    # depth_im = torch.randn(2, 1, height, width).cuda()
    for i in tqdm(range(0, 100)):
        torch.cuda.synchronize()
        torch.cuda.synchronize()
        start = time.perf_counter()
        with torch.no_grad():
            output = model(input)  # , aucx=False)
        torch.cuda.synchronize()  # wait for mm to finish
        end = time.perf_counter()
        run_time.append(end - start)
    run_time.pop(0)
    # print('Mean running time is ', np.mean(run_time))
    FPS = 1 / np.mean(run_time)
    # model_summary(model, input)
    print('#Model: {} | Runtime: {:.2f}s | FPS: {:.2f}'.format(
        model._get_name(), np.mean(run_time), FPS))
    print('-' * 90)



#!/usr/bin/env python
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-02-24 18:22:41
FilePath     : /manifast/manifast/import_pkg.py
Description  : 
LastEditTime : 2023-06-16 09:44:41
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''

import os
import re
import cv2
import sys
import math
import time
import json
import random
import shutil
import matplotlib
import numpy as np
import pandas as pd
from glob import glob
from tqdm import tqdm
from loguru import logger
import os.path as osp
from easydict import EasyDict as edict
from PIL import Image, ImageDraw
from multiprocessing import Pool
from argparse import ArgumentParser
import matplotlib.pyplot as plt
from easydict import EasyDict as edict
# 设置字体
import matplotlib.font_manager as fm
font_path = fm.findfont('SimHei')  # 查找SimHei字体的路径
plt.rcParams['font.family'] = fm.FontProperties(fname=font_path).get_name()
plt.rcParams['font.sans-serif'] = ['SimHei']
plt.rcParams['axes.unicode_minus'] = False
if not os.getcwd().split('\\')[-1].find('notebooks') == -1:
    os.chdir('../')
sys.path.append(os.getcwd())



#!/usr/bin/env python
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-02-25 09:33:08
FilePath     : /manifast/manifast/label_utils/segment/colorize_mask.py
Description  : 
LastEditTime : 2023-02-25 09:34:12
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''
from PIL import Image
import torch
import numpy as np

cityscapes_palette = [128, 64, 128, 244, 35, 232, 70, 70, 70, 102,
                      102, 156, 190, 153, 153, 153, 153, 153, 250,
                      170, 30, 220, 220, 0, 107, 142, 35, 152, 251,
                      152, 70, 130, 180, 220, 20, 60, 255, 0, 0, 0,
                      0, 142, 0, 0, 70, 0, 60, 100, 0, 80, 100, 0, 0,
                      230, 119, 11, 32]

paris_palette = [255, 255, 255, 0, 0, 255, 255, 0, 0]

road_palette = [0, 0, 0, 255, 255, 255]

austin_palette = [0, 0, 0, 255, 255, 255]

isprs_palette = [255, 255, 255, 0, 0, 255, 0,
                 255, 255, 0, 255, 0, 255, 255, 0, 255, 0, 0]
mine_palette = [0, 0, 0, 128, 0, 0, 0, 128, 0, 128, 128, 0,
                0, 0, 128, 128, 0, 128, 0, 128, 128, 128, 128,
                128, 64, 0, 0, 192, 0, 0, 64, 128, 0, 192, 128, 0]
mine_palette = [0, 64, 128,
                0, 128, 0,
                128, 0, 0,
                0, 0, 128,
                128, 128, 0,
                0, 128, 128,
                128, 0, 128,
                128, 128, 128,
                64, 0, 0,
                192, 0, 0,
                64, 128, 0,
                192, 128, 0]

# isprs_palette = [255, 255, 255, 255, 0, 0, 255, 255, 0, 0, 255, 0, 0, 255, 255, 0, 0, 255]
zero_pad = 256 * 3 - len(cityscapes_palette)
for i in range(zero_pad):
    cityscapes_palette.append(0)


# zero_pad = 256 * 3 - len(camvid_palette)
# for i in range(zero_pad):
#     camvid_palette.append(0)

def cityscapes_colorize_mask(mask):
    # mask: numpy array of the mask
    new_mask = Image.fromarray(mask.astype(np.uint8)).convert('P')
    new_mask.putpalette(cityscapes_palette)

    return new_mask


def paris_colorize_mask(mask):
    # mask: numpy array of the mask
    new_mask = Image.fromarray(mask.astype(np.uint8)).convert('P')
    new_mask.putpalette(paris_palette)

    return new_mask


def road_colorize_mask(mask):
    # mask: numpy array of the mask
    new_mask = Image.fromarray(mask.astype(np.uint8)).convert('P')
    new_mask.putpalette(road_palette)

    return new_mask


def isprs_colorize_mask(mask):
    # mask: numpy array of the mask
    new_mask = Image.fromarray(mask.astype(np.uint8)).convert('P')
    new_mask.putpalette(isprs_palette)

    return new_mask


def austin_colorize_mask(mask):
    # mask: numpy array of the mask
    new_mask = Image.fromarray(mask.astype(np.uint8)).convert('P')
    new_mask.putpalette(austin_palette)

    return new_mask


def minesegment_colorize_mask(mask):
    # mask: numpy array of the mask
    new_mask = Image.fromarray(mask.astype(np.uint8)).convert('P')
    new_mask.putpalette(mine_palette)

    return new_mask


class VOCColorize(object):
    def __init__(self, n=22):
        self.cmap = voc_color_map(22)
        self.cmap = torch.from_numpy(self.cmap[:n])

    def __call__(self, gray_image):
        size = gray_image.shape
        color_image = np.zeros((3, size[0], size[1]), dtype=np.uint8)

        for label in range(0, len(self.cmap)):
            mask = (label == gray_image)
            color_image[0][mask] = self.cmap[label][0]
            color_image[1][mask] = self.cmap[label][1]
            color_image[2][mask] = self.cmap[label][2]

        # handle void
        mask = (255 == gray_image)
        color_image[0][mask] = color_image[1][mask] = color_image[2][mask] = 255

        return color_image


def voc_color_map(N=256, normalized=False):
    def bitget(byteval, idx):
        return ((byteval & (1 << idx)) != 0)

    dtype = 'float32' if normalized else 'uint8'
    cmap = np.zeros((N, 3), dtype=dtype)
    for i in range(N):
        r = g = b = 0
        c = i
        for j in range(8):
            r = r | (bitget(c, 0) << 7 - j)
            g = g | (bitget(c, 1) << 7 - j)
            b = b | (bitget(c, 2) << 7 - j)
            c = c >> 3

        cmap[i] = np.array([r, g, b])

    cmap = cmap / 255 if normalized else cmap
    return cmap



#!/usr/bin/env python3
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-04-20 15:01:35
FilePath     : /manifast/manifast/iostream/envstream.py
Description  : 
LastEditTime : 2023-04-20 15:05:50
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''

'''
Ubuntu 避免与 ROS python 环境冲突：
conda deactivate
conda activate base 
conda config --set auto_activate_base false 
conda config --set auto_activate_base true
'''
'''
Conda 更换镜像源
# 生成配置文件 .condarc
conda config --set show_channel_urls yes
# 配置文件的目录是：~/.condarc
vim ~/.condarc
# 修改condarc内容
channels:
  - https://mirrors.tuna.tsinghua.edu.cn/anaconda/pkgs/main/
  - https://mirrors.tuna.tsinghua.edu.cn/anaconda/pkgs/free/
  - https://mirrors.tuna.tsinghua.edu.cn/anaconda/cloud/conda-forge/
  - https://mirrors.tuna.tsinghua.edu.cn/anaconda/cloud/pytorch/
ssl_verify: true
'''
'''
创建conda 虚拟环境
conda create -n torch python=3.6 #创建python3.6的torch虚拟环境
activate torch #激活虚拟环境torch

#安装ipykernel模块
conda install ipykernel
# 或
conda install jupyter notebook 
python -m ipykernel install --user --name torch --display-name "torch" #进行配置

#删除虚拟环境
conda remove -n torch --all
# 清除jupyter安装核
jupyter kernelspec list # 查看安装的内核和位置
jupyter kernelspec remove z1 #如果不正确移除该名字的kernel（假设叫做z1）或者不想要该kernel直接移除
'''
'''
第三方基础库安装
python -m pip install --upgrade pip
pip install autopep8
pip install flake8
pip install tqdm
pip install progressbar
pip install Pillow
pip install opencv-python==4.2.0.32
pip install h5py==2.10.0
pip install pandas
pip install labelme==3.16.5
pip install matplotlib
conda install yaml
conda install easydict
'''


def install_pkg():
    pass



#!/usr/bin/env python
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-02-24 21:39:35
FilePath     : /manifast/manifast/iostream/__init__.py
Description  : 
LastEditTime : 2023-04-20 15:01:48
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''
from .filestream import *
from .imagestream import *
from .pdstream import *
from .pltstream import *
from .processsing import *
from .logstream import *
from .envstream import *



#!/usr/bin/env python3
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-02-27 09:41:52
FilePath     : /manifast/manifast/gui_utils/__init__.py
Description  : 
LastEditTime : 2023-02-27 09:41:55
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''



#!/usr/bin/env python
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-02-24 16:57:43
FilePath     : /manifast/manifast/iostream/filestream.py
Description  : 
LastEditTime : 2023-06-18 14:15:49
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''

import os
import time
import json


def get_files(file_dir='', extention=['.jpg'], in_line=None, out_line=None):
    """
     description: 获取指定文件夹下的所有指定文件类型的文件，到List
     param       {*} file_dir  指定的迭代检索的文件夹
     param       {*} tail_name 迭代检测的文件类型
     return      {*} 包含所有文件绝对路径的list
    """
    if not isinstance(extention, list):
        extention = [extention]
    path_list = []
    if in_line:
        if out_line:
            for (dirpath, dirnames, filenames) in os.walk(file_dir):
                for filename in filenames:
                    if (os.path.splitext(filename)[1] in extention) and (in_line in os.path.join(dirpath, filename)) and (out_line not in os.path.join(dirpath, filename)):
                        path_list.append(os.path.join(dirpath, filename))
        else:
            for (dirpath, dirnames, filenames) in os.walk(file_dir):
                for filename in filenames:
                    if (os.path.splitext(filename)[1] in extention) and (in_line in os.path.join(dirpath, filename)):
                        path_list.append(os.path.join(dirpath, filename))
    else:
        if out_line:
            for (dirpath, dirnames, filenames) in os.walk(file_dir):
                for filename in filenames:
                    if (os.path.splitext(filename)[1] in extention) and (out_line not in os.path.join(dirpath, filename)):
                        path_list.append(os.path.join(dirpath, filename))
        else:
            for (dirpath, dirnames, filenames) in os.walk(file_dir):
                for filename in filenames:
                    if (os.path.splitext(filename)[1] in extention):
                        path_list.append(os.path.join(dirpath, filename))
    return path_list


def make_path_dirs(path):
    if '.' in os.path.basename(path):
        os.makedirs(os.path.dirname(path), exist_ok=True)
    else:
        os.makedirs(path, exist_ok=True)


def write_dict2json(data={}, save_path='./results.json'):
    """
     description: 将data保存到指定的json文件下
     param       {*} data 
     param       {*} save_path
     return      {*}
    """
    with open(save_path, 'w', encoding='utf-8') as f:
        json.dump(data, f, indent=4)


def read_json_data(readPath):
    with open(readPath, "r", encoding='utf-8') as f:
        dict = json.load(f)
    return dict


def read_txt_data(txt_path):
    constants = []
    with open(txt_path, 'r') as file_to_read:
        while True:
            lines = file_to_read.readline()  # 整行读取数据
            if not lines:
                break
            # 将整行数据分割处理，如果分割符是空格，括号里就不用传入参数，如果是逗号， 则传入‘，'字符。
            # p_tmp = [i for i in lines.split(' ')]
            constants.append(lines.strip('\n'))  # 添加新读取的数据
            # Efield.append(E_tmp)
            pass
    return constants


def write_list2txt(datas=[], save_path='./results.txt'):
    """
     description: 将data保存到指定的json文件下
     param       {*} data 
     param       {*} save_path
     return      {*}
    """
    with open(save_path, 'w') as f:
        pass
    for data in datas:
        if isinstance(data, int):
            with open(save_path, 'a') as f:
                f.write(str(data)+'\n')
        elif isinstance(data, str):
            with open(save_path, 'a') as f:
                f.write(str(data)+'\n')
        elif isinstance(data, list):
            for dt in data:
                with open(save_path, 'a') as f:
                    f.write(str(dt)+',')
            with open(save_path, 'a') as f:
                f.write('\n')
        elif isinstance(data, set):
            for dt in data:
                with open(save_path, 'a') as f:
                    f.write(str(dt)+',')
            with open(save_path, 'a') as f:
                f.write('\n')
        else:
            for dt in data:
                with open(save_path, 'a') as f:
                    f.write(str(dt)+',')
            with open(save_path, 'a') as f:
                f.write('\n')


def get_time():
    return time.strftime('%Y%m%d', time.localtime(time.time()))+time.strftime('_%H%M%S', time.localtime(time.time()))


def get_current_daytime():
    return time.strftime('%Y%m%d', time.localtime(time.time()))


def get_current_clocktime():
    return time.strftime('%H%M%S', time.localtime(time.time()))



import flet as ft


def main(page: ft.Page):
    def on_keyboard(e: ft.KeyboardEvent):
        page.add(
            ft.Text(
                f"Key: {e.key}, Shift: {e.shift}, Control: {e.ctrl}, Alt: {e.alt}, Meta: {e.meta}"
            )
        )

    page.on_keyboard_event = on_keyboard
    page.add(
        ft.Text("Press any key with a combination of CTRL, ALT, SHIFT and META keys...")
    )


ft.app(target=main)



#!/usr/bin/env python
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-02-24 21:54:44
FilePath     : /manifast/manifast/iostream/pdstream.py
Description  : 
LastEditTime : 2023-04-04 15:04:06
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''
import pandas as pd


def get_df_types(dataframe, key):
    '''
    description: 获取dataframe中某列key的所有类型
    param       {*} dataframe DF数据表
    param       {*} key 列名
    return      {*} 类型 list和类型数量
    '''
    return list(dataframe[[key]].drop_duplicates()[key]), len(list(dataframe[[key]].drop_duplicates()[key]))



#!/usr/bin/env python
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-02-24 20:55:39
FilePath     : /manifast/manifast/model_utils/grad_cam.py
Description  : 
LastEditTime : 2023-02-25 09:18:34
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''

import numpy as np
import cv2
from torchvision.models import mobilenet_v3_small


def visCaminImage(img, mask):
    heatmap = cv2.applyColorMap(
        np.uint8(255*mask), cv2.COLORMAP_JET)  # 利用色彩空间转换将heatmap凸显
    heatmap = np.float32(heatmap)/255  # 归一化
    cam = heatmap + np.float32(img)  # 将heatmap 叠加到原图
    cam = cam / np.max(cam)
    # cv2.imwrite('GradCam_test.jpg', np.uint8(255 * cam))  # 生成图像

    cam = cam[:, :, ::-1]  # BGR > RGB
    return np.uint8(255*cam)


class GradCAM(object):
    """
    1: 网络不更新梯度,输入需要梯度更新
    2: 使用目标类别的得分做反向传播
    """

    def __init__(self, net, layer_name):
        self.net = net
        self.layer_name = layer_name
        self.feature = None
        self.gradient = None
        self.net.eval()
        self.handlers = []
        self._register_hook()

    def _get_features_hook(self, module, input, output):
        self.feature = output
#         print("feature shape:{}".format(output.size()))

    def _get_grads_hook(self, module, input_grad, output_grad):
        """
        :param input_grad: tuple, input_grad[0]: None
                                   input_grad[1]: weight
                                   input_grad[2]: bias
        :param output_grad:tuple,长度为1
        :return:
        """
        self.gradient = output_grad[0]

    def _register_hook(self):
        for (name, module) in self.net.named_modules():
            if name == self.layer_name:
                self.handlers.append(
                    module.register_forward_hook(self._get_features_hook))
                self.handlers.append(
                    module.register_backward_hook(self._get_grads_hook))

    def remove_handlers(self):
        for handle in self.handlers:
            handle.remove()

    def __call__(self, inputs, cls_num=39):
        """
        :param inputs: [1,3,H,W]
        :param cls_num: class id
        :return:
        """
        self.net.zero_grad()
        x, y = inputs
        output = self.net(x, y)  # [1,num_classes]
        if cls_num is None:
            cls_num = np.argmax(output.cpu().data.numpy())
        target = output[0][cls_num][0][0]

        target.backward()

        gradient = self.gradient[0].cpu().data.numpy()  # [C,H,W]
        weight = np.mean(gradient, axis=(1, 2))  # [C]

        feature = self.feature[0].cpu().data.numpy()  # [C,H,W]

        cam = feature * weight[:, np.newaxis, np.newaxis]  # [C,H,W]
        cam = np.sum(cam, axis=0)  # [H,W]
        cam = np.maximum(cam, 0)  # ReLU

        # 数值归一化
        cam -= np.min(cam)
        cam /= np.max(cam)
        # resize to 224*224
        cam = cv2.resize(cam, (640, 480))
        return cam


class GradCamPlusPlus(GradCAM):
    def __init__(self, net, layer_name):
        """
         description: 
         param       {*} net model模型
         param       {*} layer_name     
         return      {*}
        """
        super(GradCamPlusPlus, self).__init__(net, layer_name)

    def __call__(self, inputs, cls_num):
        """
        :param inputs: [1,3,H,W] 
        :param cls_num: class num
        :return:
        """
        self.feature = None
        self.gradient = None
        self.net.zero_grad()
        x, y = inputs
        h, w = x.shape[-2:]
        output = self.net(inputs)  # [1,num_classes]
        if cls_num is None:
            cls_num = np.argmax(output.cpu().data.numpy())

        target = output[0][0][cls_num][0][0]
#         print(target)
        target.backward()

        gradient = self.gradient[0].cpu().data.numpy()  # [C,H,W]
        gradient = np.maximum(gradient, 0.)  # ReLU
        indicate = np.where(gradient > 0, 1., 0.)  # 示性函数
        norm_factor = np.sum(gradient, axis=(1, 2))  # [C]归一化
        for i in range(len(norm_factor)):
            norm_factor[i] = 1. / \
                norm_factor[i] if norm_factor[i] > 0. else 0.  # 避免除零
        alpha = indicate * norm_factor[:, np.newaxis, np.newaxis]  # [C,H,W]

        # [C]  alpha*ReLU(gradient)
        weight = np.sum(gradient * alpha, axis=(1, 2))

        feature = self.feature[0].cpu().data.numpy()  # [C,H,W]

        cam = feature * weight[:, np.newaxis, np.newaxis]  # [C,H,W]
        cam = np.sum(cam, axis=0)  # [H,W]
        # cam = np.maximum(cam, 0)  # ReLU

        # 数值归一化
        cam -= np.min(cam)
        cam /= np.max(cam)
        # resize to 224*640
        cam = cv2.resize(cam, (w, h))
        return cam


def alyModelFeatureCam(image, model=mobilenet_v3_small(), feats=['ident1',
                                                                 'ident2',
                                                                 'ident3',
                                                                 'ident4',
                                                                 'ident5', ]):

    grad_cam = GradCamPlusPlus(
        net=model, layer_name='encoder_depth.layer4.2.act')
    mask = grad_cam(inputs=(image), cls_num=39)
    img = image.copy()
    img = np.float32(cv2.resize(img, (640, 480))) / 255.
    visCaminImage(img, mask)



#!/usr/bin/env python
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-02-24 23:40:00
FilePath     : /manifast/manifast/label_utils/detect/gen_split_data.py
Description  : 
LastEditTime : 2023-02-25 10:53:26
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''
from convert_label import xywh2xyxy
from manifast import *
import numpy as np


def genGTDatabase(image_path, txt_path, num2label_dict={}, save_crop_dir='./tmp/database/det'):
    '''
    description: 生成ground truth database
    param       {*} image_path 图像路径
    param       {*} txt_path 标注路径
    param       {*} save_crop_dir 数据库保存路径
    return      {*} crop image path list
    '''
    im_pil = readImagFilebyPillow(image_path)
    poses = readTxtFile(txt_path)
    if not num2label_dict:
        classes = list(set([int(line.split()[0]) for line in poses]))
        for i in classes:
            num2label_dict[i] = str(i)
    width, height = im_pil.size
    i = 0
    crop_img_paths = []
    for l in poses:
        line = l.split()
        xywh = np.array(
            list(map(float, line[1:]))) * np.array([width, height, width, height])
        [xyxy] = xywh2xyxy(np.array([xywh]))
        crop_image_name = os.path.basename(image_path)
        new_name = crop_image_name.replace('.jpg', f'-{i}.jpg')
        new_path = os.path.join(
            save_crop_dir, num2label_dict[int(line[0])], new_name)
        makePathDirs(new_path)
        im_crop = im_pil.crop(xyxy)
        im_crop.save(new_path)
        crop_img_paths.append(new_path)
        i += 1
    return crop_img_paths


if __name__ == '__main__':
    image_path = '/home/knight/waytous/dataset/minedatset/images/sence_0_000106.jpg'
    txt_path = image_path.replace(
        'images', 'labelBoxs').replace('.jpg', '.txt')
    genGTDatabase(image_path, txt_path)



#!/usr/bin/env python
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-02-25 09:28:57
FilePath     : /manifast/manifast/label_utils/segment/convert_utils.py
Description  : 
ref https://github.dev/guchengxi1994/mask2json
LastEditTime : 2023-02-25 10:17:05
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''
import cv2
import numpy as np


def cropRotateBox(cnt, img):

    rect = cv2.minAreaRect(cnt)
    box = cv2.boxPoints(rect)
    box = np.int0(box)

    width = int(rect[1][0])
    height = int(rect[1][1])
    src_pts = box.astype("float32")
    dst_pts = np.array([[0, height - 1],
                        [0, 0],
                        [width - 1, 0],
                        [width - 1, height - 1]], dtype="float32")

    M = cv2.getPerspectiveTransform(src_pts, dst_pts)

    warped = cv2.warpPerspective(img, M, (width, height))
    return warped


def rotateImagebyHeightLonger(img):
    h, w, _ = img.shape

    if h > w:
        '''getRotationMatrix2D only rotate kepp same ratio'''
        # cv2.getRotationMatrix2D出来的图特别小，是在原始图上截取的，
        # 也就是说要先对原图做padding，不然旋转后超出原始图像的像素会不被保留，所以最后改成了np.rot90
        # center = (w / 2, h / 2)
        # M = cv2.getRotationMatrix2D(center,90,1)
        # rotate_img = cv2.warpAffine(img,M,(w,h))
        # h_stack = np.hstack((img, rotate_img))
        rotate_img = np.rot90(img, 1)
        # cv2.imshow('img', rotate_img)
        # cv2.waitKey()
        return rotate_img
    else:
        return img



#!/usr/bin/env python
# coding=utf-8
'''
brief        :
Author       : knightdby knightdby@163.com
Date         : 2023-02-24 23:21:03
FilePath     : /manifast/manifast/label_utils/detect/vis_label.py
Description  :
LastEditTime : 2023-02-25 11:05:01
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''
from manifast import *
from convert_label import xywh2xyxy
import cv2
import random
import colorsys
import seaborn as sn


def drawBbox(image, bboxes, classes_name, show_label=True):
    num_classes = len(classes_name)
    image_h, image_w, _ = image.shape
    hsv_tuples = [(1.0 * x / num_classes, 1., 1.) for x in range(num_classes)]
    colors = list(map(lambda x: colorsys.hsv_to_rgb(*x), hsv_tuples))
    colors = list(
        map(lambda x: (int(x[0] * 255), int(x[1] * 255), int(x[2] * 255)), colors))

    random.seed(0)
    random.shuffle(colors)
    random.seed(None)

    have_classes = []

    out_boxes, out_scores, out_classes, num_boxes = bboxes
    for i in range(num_boxes[0]):
        if int(out_classes[0][i]) < 0 or int(out_classes[0][i]) > num_classes:
            continue
        coor = out_boxes[0][i]
        coor[0] = int(coor[0] * image_h)
        coor[2] = int(coor[2] * image_h)
        coor[1] = int(coor[1] * image_w)
        coor[3] = int(coor[3] * image_w)

        fontScale = 0.5
        score = out_scores[0][i]
        class_ind = int(out_classes[0][i])
        bbox_color = colors[class_ind]
        bbox_thick = int(0.6 * (image_h + image_w) / 600)
        c1, c2 = (int(coor[1]), int(coor[0])), (int(coor[3]), int(coor[2]))
        cv2.rectangle(image, c1, c2, bbox_color, bbox_thick)

        if show_label:
            bbox_mess = '%s: %.2f' % (classes_name[class_ind], score)
            have_classes.append(classes_name[class_ind])
            t_size = cv2.getTextSize(
                bbox_mess, 0, fontScale, thickness=bbox_thick // 2)[0]
            c3 = (c1[0] + t_size[0], c1[1] - t_size[1] - 3)
            cv2.rectangle(image, c1, (c3[0], c3[1]), bbox_color, -1)  # filled

            cv2.putText(image, bbox_mess, (c1[0], c1[1] - 2), cv2.FONT_HERSHEY_SIMPLEX,
                        fontScale, (0, 0, 0), bbox_thick // 2, lineType=cv2.LINE_AA)
    return image, have_classes


def visBoxLabel(image_path, txt_path, label2num_dict={}):
    '''
    description: 可视化 box 标签
    param       {*} image_path 图像路径
    param       {*} txt_path label 路径
    return      {*} 可视化图
    '''
    image = readImageFilebyCv(image_path)
    poses = readTxtFile(txt_path)
    if not label2num_dict:
        for i in range(30):
            label2num_dict[str(i)] = i
    out_scores = []
    out_boxes = []
    out_classes = []
    for line in poses:
        pos = [float(i) for i in line.split(' ')]
        # ['1' '0.724877' '0.309082' '0.073938' '0.086914']
        [box] = xywh2xyxy(np.array([pos[1:]]))
        out_scores.append(0.99)
        [xmin, ymin, xmax, ymax] = box
        box = [ymin, xmin, ymax, xmax]
        out_boxes.append(box)
        out_classes.append(int(pos[0]))
    bboxes = [out_boxes], [out_scores], [out_classes], [len(poses)]
    drawBbox(image, bboxes, classes_name=list(
        label2num_dict.keys()), show_label=True)
    return image


def visBoxDistribution(box_paths, save_path='./tmp/class_distri.jpg'):
    '''
    description: 可视化 box 概率分布图
    param       {*} box_paths label path list
    param       {*} save_path 结果保存路径
    return      {*} null
    '''
    boxs = []  # 标注框xywh/size
    classes_id = []  # 类别名的索引
    for file in tqdm(box_paths):
        with open(file, 'r') as f:
            for l in f.readlines():
                # ['11' '0.724877' '0.309082' '0.073938' '0.086914']
                line = l.split()
                classes_id.append([int(line[0])])
                boxs.append(list(map(float, line[1:])))
    shapes = np.array(boxs)
    ids = np.array(classes_id)
    labels = np.hstack((ids, shapes))

    c, b = labels[:, 0], labels[:, 1:].transpose()  # classes, boxes
    nc = int(c.max() + 1)  # number of classes
    x = pd.DataFrame(b.transpose(), columns=['x', 'y', 'width', 'height'])
    ful = pd.DataFrame(labels, columns=['class', 'x', 'y', 'width', 'height'])
    sn.pairplot(ful, corner=True, diag_kind='auto', kind='hist',
                diag_kws=dict(bins=50), plot_kws=dict(pmax=0.9))
    makePathDirs(save_path)
    plt.savefig(save_path, dpi=200)
    plt.show()


if __name__ == "__main__":
    image_path = '/home/knight/waytous/dataset/minedatset/images/sence_0_000106.jpg'
    txt_path = image_path.replace(
        'images', 'labelBoxs').replace('.jpg', '.txt')
    labeled_image = visBoxLabel(image_path, txt_path)
    save_path = './tmp/det/labeled_img.jpg'
    makePathDirs(save_path)
    cv2.imwrite(save_path, labeled_image[:, :, ::-1])
    # txt_paths = getFileList(
    #     '/home/knight/waytous/dataset/minedatset/labelBoxs', '.txt')
    # print(txt_paths[0])
    # visBoxDistribution(txt_paths)



#!/usr/bin/env python
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-02-24 23:20:35
FilePath     : /manifast/manifast/label_utils/detect/convert_label.py
Description  : 
LastEditTime : 2023-02-25 08:39:03
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''
import numpy as np


def convertRawSize(size, box):
    dw = 1. / size[0]
    dh = 1. / size[1]
    x = (box[0] + box[1]) / 2.0
    y = (box[2] + box[3]) / 2.0
    w = box[1] - box[0]
    h = box[3] - box[2]
    x = x * dw
    w = w * dw
    y = y * dh
    h = h * dh
    return (x, y, w, h)


def xywh2xyxy(x):
    # Convert nx4 boxes from [x, y, w, h] to [x1, y1, x2, y2] where xy1=top-left, xy2=bottom-right
    y = np.copy(x)
    y[:, 0] = x[:, 0] - x[:, 2] / 2  # top left x
    y[:, 1] = x[:, 1] - x[:, 3] / 2  # top left y
    y[:, 2] = x[:, 0] + x[:, 2] / 2  # bottom right x
    y[:, 3] = x[:, 1] + x[:, 3] / 2  # bottom right y
    return y



#!/usr/bin/env python
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-02-24 21:58:02
FilePath     : /manifast/manifast/iostream/processsing.py
Description  : 
LastEditTime : 2023-05-08 10:31:40
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''
from multiprocessing import Pool
from tqdm import tqdm
from .logstream import LOG_INFO


def run(image_path):
    print(image_path, end='')
    return (image_path)


def run_multi_threading(data_list, func):
    '''
    description: 多线程运行函数
    param       {*} data_list 所要处理数据 list
    param       {*} func 处理单条数据函数
    return      {*} 单条数据函数的返回值 list
    '''

    LOG_INFO(f'Data_list size: {len(data_list)}')
    with Pool(processes=10) as p:
        processed_data_list = list(
            tqdm(p.imap(func, data_list), total=len(data_list), desc='run multithread'))
    return processed_data_list


if __name__ == '__main__':
    data_list = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13,
                 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24,
                 25, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35,
                 36, 37, 38, 39, 40, 41, 42, ]
    func = run_multi_threading(data_list, run)
    print(func)




#!/usr/bin/env python
# coding=utf-8
"""
 Description  : 
 
 Author       : Ding Baiyong(Knight) && knightdby@163.com
 Date         : 2022-05-11 10:24:00
 LastEditTime : 2022-05-12 19:04:36
 LastEditors  : Ding Baiyong(Knight)
 FilePath     : /segmentation/utils/fps_test/eval_forward_time.py
"""
import time
import torch
import torch.backends.cudnn as cudnn

from argparse import ArgumentParser
from builders.model_builder import build_model


def compute_speed(model, input_size, device, iteration=100):
    torch.cuda.set_device(device)
    cudnn.benchmark = True

    model.eval()
    model = model.cuda()

    input = torch.randn(*input_size, device=device)

    for _ in range(50):
        model(input)

    print('=========Eval Forward Time=========')
    torch.cuda.synchronize()
    t_start = time.time()
    for _ in range(iteration):
        model(input)
    torch.cuda.synchronize()
    elapsed_time = time.time() - t_start

    speed_time = elapsed_time / iteration * 1000
    fps = iteration / elapsed_time

    print('Elapsed Time: [%.2f s / %d iter]' % (elapsed_time, iteration))
    print('Speed Time: %.2f ms / iter   FPS: %.2f' % (speed_time, fps))
    return speed_time, fps


if __name__ == '__main__':
    parser = ArgumentParser()

    parser.add_argument("--size", type=str, default="512,512",
                        help="input size of model")
    parser.add_argument('--num-channels', type=int, default=3)
    parser.add_argument('--batch-size', type=int, default=1)
    parser.add_argument('--classes', type=int, default=3)
    parser.add_argument('--iter', type=int, default=100)
    parser.add_argument('--model', type=str, default='NFSNet')
    parser.add_argument("--gpus", type=str, default="0",
                        help="gpu ids (default: 0)")
    args = parser.parse_args()

    h, w = map(int, args.size.split(','))
    model = build_model(args.model, num_classes=args.classes)
    compute_speed(model, (args.batch_size, args.num_channels,
                  h, w), int(args.gpus), iteration=args.iter)



#!/usr/bin/env python3
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-04-04 15:20:13
FilePath     : /manifast/manifast/iostream/logstream.py
Description  : 
LastEditTime : 2023-04-06 10:01:32
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''

from loguru import logger


def logger_init():
    logger.add('runtime_{time}.log', rotation="500 MB")


def LOG_INFO(msg):
    logger.info(msg)


def LOG_WARNING(msg):
    logger.warning(msg)


def LOG_ERROR(msg):
    logger.error(msg)


if __name__ == '__main__':
    LOG_INFO('start')
    LOG_WARNING('start')
    LOG_ERROR('start')
    new_level = logger.level("SNAKY", no=38, color="<yellow>", icon="🐍")
    logger.log("SNAKY", "Here we go!")



#!/usr/bin/env python
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-02-24 23:21:14
FilePath     : /manifast/manifast/label_utils/segment/vis_label.py
Description  : 
LastEditTime : 2023-02-25 22:17:11
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''
import cv2
from vis_utils import draw_label
from manifast import *


def visMaskLabel(image_path, label_path, label2num_dict={}, color_map=None):
    image = readImageFilebyCv(image_path)
    label_img = readImageFilebyCv(label_path)
    if not label2num_dict:
        for i in range(np.max(label_img)+1):
            label2num_dict[str(i)] = i
    mask_img = draw_label(
        label_img,
        image[:, :, ::-1],
        list(label2num_dict.keys()), color_map)
    return mask_img


def visMaskDistribution():
    pass


if __name__ == "__main__":
    image_path = '/home/knight/waytous/dataset/minedatset/images/sence_0_000106.jpg'
    idlabel_path = image_path.replace(
        'images', 'labelIds').replace('.jpg', '.png')
    mask_img = visMaskLabel(image_path, idlabel_path)
    save_path = './tmp/seg/masked_img.jpg'
    makePathDirs(save_path)
    cv2.imwrite(save_path, mask_img[:, :, ::-1])



#!/usr/bin/env python3
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-02-25 11:10:37
FilePath     : /manifast/manifast/label_utils/segment/vis_utils.py
Description  : 
LastEditTime : 2023-02-25 22:20:50
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''
import matplotlib.pyplot as plt
import io
import os.path as osp

import numpy as np
import PIL.Image
import PIL.ImageDraw
import PIL.ImageFont


def label_colormap(N=256):

    def bitget(byteval, idx):
        return ((byteval & (1 << idx)) != 0)

    cmap = np.zeros((N, 3))
    for i in range(0, N):
        id = i
        r, g, b = 0, 0, 0
        for j in range(0, 8):
            r = np.bitwise_or(r, (bitget(id, 0) << 7 - j))
            g = np.bitwise_or(g, (bitget(id, 1) << 7 - j))
            b = np.bitwise_or(b, (bitget(id, 2) << 7 - j))
            id = (id >> 3)
        cmap[i, 0] = r
        cmap[i, 1] = g
        cmap[i, 2] = b
    cmap = cmap.astype(np.float32) / 255
    return cmap


def _validate_colormap(colormap, n_labels):
    if colormap is None:
        colormap = label_colormap(n_labels)
    else:
        assert colormap.shape == (colormap.shape[0], 3), \
            'colormap must be sequence of RGB values'
        assert 0 <= colormap.min() and colormap.max() <= 1, \
            'colormap must ranges 0 to 1'
    return colormap


# similar function as skimage.color.label2rgb
def label2rgb(
    lbl, img=None, n_labels=None, alpha=0.5, thresh_suppress=0, colormap=None,
):
    if n_labels is None:
        n_labels = len(np.unique(lbl))

    colormap = _validate_colormap(colormap, n_labels)
    colormap = (colormap * 255).astype(np.uint8)

    lbl_viz = colormap[lbl]
    lbl_viz[lbl == -1] = (0, 0, 0)  # unlabeled

    if img is not None:
        img_gray = PIL.Image.fromarray(img).convert('LA')
        img_gray = np.asarray(img_gray.convert('RGB'))
        # img_gray = cv2.cvtColor(img, cv2.COLOR_RGB2GRAY)
        # img_gray = cv2.cvtColor(img_gray, cv2.COLOR_GRAY2RGB)
        lbl_viz = alpha * lbl_viz + (1 - alpha) * img_gray
        lbl_viz = lbl_viz.astype(np.uint8)

    return lbl_viz


def draw_label(label, img=None, label_names=None, colormap=None, **kwargs):
    """Draw pixel-wise label with colorization and label names.

    label: ndarray, (H, W)
        Pixel-wise labels to colorize.
    img: ndarray, (H, W, 3), optional
        Image on which the colorized label will be drawn.
    label_names: iterable
        List of label names.
    """

    backend_org = plt.rcParams['backend']
    plt.switch_backend('agg')

    plt.subplots_adjust(left=0, right=1, top=1, bottom=0,
                        wspace=0, hspace=0)
    plt.margins(0, 0)
    plt.gca().xaxis.set_major_locator(plt.NullLocator())
    plt.gca().yaxis.set_major_locator(plt.NullLocator())

    if label_names is None:
        label_names = [str(l) for l in range(label.max() + 1)]

    colormap = _validate_colormap(colormap, len(label_names))

    label_viz = label2rgb(
        label, img, n_labels=len(label_names), colormap=colormap, **kwargs
    )
    plt.imshow(label_viz)
    plt.axis('off')

    plt_handlers = []
    plt_titles = []
    for label_value, label_name in enumerate(label_names):
        if label_value not in label:
            continue
        fc = colormap[label_value]
        p = plt.Rectangle((0, 0), 1, 1, fc=fc)
        plt_handlers.append(p)
        plt_titles.append('{value}: {name}'
                          .format(value=label_value, name=label_name))
    plt.legend(plt_handlers, plt_titles, loc='lower right', framealpha=.5)

    f = io.BytesIO()
    plt.savefig(f, bbox_inches='tight', pad_inches=0)
    plt.cla()
    plt.close()

    plt.switch_backend(backend_org)

    out_size = (label_viz.shape[1], label_viz.shape[0])
    out = PIL.Image.open(f).resize(out_size, PIL.Image.BILINEAR).convert('RGB')
    out = np.asarray(out)
    return out


def draw_instances(
    image=None,
    bboxes=None,
    labels=None,
    masks=None,
    captions=None,
):
    import matplotlib

    # TODO(wkentaro)
    assert image is not None
    assert bboxes is not None
    assert labels is not None
    assert masks is None
    assert captions is not None

    viz = PIL.Image.fromarray(image)
    draw = PIL.ImageDraw.ImageDraw(viz)

    font_path = osp.join(
        osp.dirname(matplotlib.__file__),
        'mpl-data/fonts/ttf/DejaVuSans.ttf'
    )
    font = PIL.ImageFont.truetype(font_path)

    colormap = label_colormap(255)
    for bbox, label, caption in zip(bboxes, labels, captions):
        color = colormap[label]
        color = tuple((color * 255).astype(np.uint8).tolist())

        xmin, ymin, xmax, ymax = bbox
        draw.rectangle((xmin, ymin, xmax, ymax), outline=color)
        draw.text((xmin, ymin), caption, font=font)

    return np.asarray(viz)



#!/usr/bin/env python
# coding=utf-8
'''
brief        :
Author       : knightdby knightdby@163.com
Date         : 2023-02-24 21:40:04
FilePath     : /manifast/manifast/iostream/imagestream.py
Description  :
LastEditTime : 2023-04-09 11:40:59
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''


import os
import cv2
import numpy as np
from PIL import Image, ImageDraw, ImageFont


def read_image_cv(path):
    '''
    description: 读取图像数据
    param       {*} path 图像路径
    return      {*} 图像数据
    '''

    img = cv2.imread(path, cv2.IMREAD_UNCHANGED)
    if img.ndim == 3:
        img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    return img


def read_image_pillow(path):
    with open(path, "rb") as f:
        img = Image.open(f)
        return img.convert("RGB")


def cv2image_add_mask(img, mask):
    """
     description: 将彩色图像与mask相叠加
     param       {*} img h*w*c
     param       {*} mask h*w
     return      {*} 叠加效果图
    """
    if mask.ndim == 3:
        mask = mask.squeeze()  # 去掉batch維度
    mask = mask.astype(np.uint8)
    display_mask = np.zeros_like(img)
    display_mask[mask == 1, 0] = 255
    masked = cv2.add(img, np.zeros(
        np.shape(img), dtype=np.uint8), mask=mask)
    masked = cv2.addWeighted(img, 0.5, display_mask, 0.5, 0)
    return masked


def cv2image_add_text(img, text, left, top, textColor=(0, 255, 0), textSize=20, font="Times_New_Roman_Bold"):
    '''
    description: 绘制文字至cv 图像中
    param       {*} img 绘制图像
    param       {*} text 文字内容
    param       {*} left x 位置
    param       {*} top y 位置
    param       {*} textColor 字体颜色
    param       {*} textSize 字体大小
    param       {*} font 字体
    return      {*} 绘制文字之后的 cv 图像
    '''
    if (isinstance(img, np.ndarray)):  # 判断是否OpenCV图片类型
        img = Image.fromarray(cv2.cvtColor(img, cv2.COLOR_BGR2RGB))
    draw = ImageDraw.Draw(img)
    fontText = ImageFont.truetype(
        font, textSize, encoding="utf-8")
    draw.text((left, top), text, textColor, font=fontText)
    return cv2.cvtColor(np.asarray(img), cv2.COLOR_RGB2BGR)



#!/usr/bin/env python
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-02-24 23:21:13
FilePath     : /manifast/manifast/label_utils/segment/convert_label.py
Description  : 
LastEditTime : 2023-05-19 14:32:06
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''
import json
import numpy as np
from manifast import *


def label_to_onehot(label, colormap):
    """  
    Converts a segmentation label (H, W, C) to (H, W, K) where the last dim is a one  
    hot encoding vector, C is usually 1 or 3, and K is the number of class.  
    """
    semantic_map = []
    for colour in colormap:
        equality = np.equal(label, colour)
        class_map = np.all(equality, axis=-1)
        semantic_map.append(class_map)
    semantic_map = np.stack(semantic_map, axis=-1).astype(np.float32)
    return semantic_map


def onehot_to_label(semantic_map, colormap):
    """  
    Converts a mask (H, W, K) to (H, W, C)  
    """
    x = np.argmax(semantic_map, axis=-1)
    colour_codes = np.array(colormap)
    label = np.uint8(colour_codes[x.astype(np.uint8)])
    return label


def onehot2mask(semantic_map):
    """  
    Converts a mask (K, H, W) to (H,W)  
    """
    _mask = np.argmax(semantic_map, axis=0).astype(np.uint8)
    return _mask


def mask2onehot(mask, num_classes):
    """  
    Converts a segmentation mask (H,W) to (K,H,W) where the last dim is a one  
    hot encoding vector  

    """
    semantic_map = [mask == i for i in range(num_classes)]
    return np.array(semantic_map).astype(np.uint8)


def convertLabel2LabelmeJson(image_path, json_path, label_save_path):
    '''
    description: 转为标准 Labelme 格式的标注Json文件
    param       {*} image_path 图像路径
    param       {*} json_path label 路径（.json）
    param       {*} label_save_path 标准 label 保存路径
    return      {*} mask 数量
    '''
    with open(json_path, 'r', encoding='utf-8') as load_f:
        label_content = json.load(load_f)
    json_dict = {"version": "4.6.0", "flags": {}, "shapes": [],
                 "imagePath": "", "imageData": None,
                 "imageHeight": 0, "imageWidth": 0}
    json_dict["imagePath"] = image_path
    json_dict["imageHeight"] = label_content['info']['height']
    json_dict["imageWidth"] = label_content['info']['width']
    label_mask_num = 0
    for obj in label_content['dataList']:
        label_mask_num += 1
        img_mark_inf = {"label": "", "line_color": None, "fill_color": None,
                        "points": [], "shape_type": "polygon", "flags": {}}
        img_mark_inf["label"] = obj['label']
        img_mark_inf["shape_type"] = obj['shapeType']
        img_mark_inf["label"] = obj['label']
        for point in obj['coordinates'][0]:
            img_mark_inf["points"].append(point)
        json_dict["shapes"].append(img_mark_inf)
    make_path_dirs(label_save_path)
    with open(label_save_path, "w") as out_json:
        json.dump(json_dict, out_json, ensure_ascii=False, indent=2)
    return label_mask_num


def convertLabelJson2IdImage(json_path, label2num_dict):
    '''
    description: 将 json 标准文件转为 label image
    param       {*} json_path label 路径
    param       {*} label2num_dict label 和 id 对应关系 dict
    return      {*} 标签 image: labelIds
    '''
    jsondata = json.load(open(json_path))
    img = cv2.imread(os.path.join(
        os.path.dirname(json_path), jsondata['imagePath']))
    shapes = jsondata['shapes']
    gt_mask = np.zeros(img.shape, dtype='uint8')
    # 生成顺序，首先绘制sky天空，->背景土墙->道路->挡墙->其他标签
    # for i in shapes_raw:
    #     label = i['label']
    #     points = i['points']
    #     if label == 'sky':
    #         points = np.array(points).reshape(-1, 1, 2).astype(int)
    #         gt_mask = cv2.fillPoly(
    #             gt_mask, [points], label2num_dict[label])
    for i in shapes:
        label = i['label']
        points = i['points']
        if label == 'sky':
            points = np.array(points).reshape(-1, 1, 2).astype(int)
            # 使用mask面积作为阈值，删除小面积的天空mask，将其还原为0
            # if cv2.contourArea(points) > (1920*100):
            gt_mask = cv2.fillPoly(
                gt_mask, [points], label2num_dict[label])
    for i in shapes:
        label = i['label']
        points = i['points']
        if label == 'backgroundwall':
            points = np.array(points).reshape(-1, 1, 2).astype(int)
            gt_mask = cv2.fillPoly(
                gt_mask, [points], label2num_dict[label])
    # for i in shapes_raw:
    #     label = i['label']
    #     points = i['points']
    #     if label == 'Backgroundwall':
    #         points = np.array(points).reshape(-1, 1, 2).astype(int)
    #         gt_mask = cv2.fillPoly(
    #             gt_mask, [points], label2num_dict[label])
    for i in shapes:
        label = i['label']
        points = i['points']
        if label == 'road':
            points = np.array(points).reshape(-1, 1, 2).astype(int)
            gt_mask = cv2.fillPoly(
                gt_mask, [points], (label2num_dict[label], 0, 0))
    # for i in shapes_raw:
    #     label = i['label']
    #     points = i['points']
    #     if label == 'Road':
    #         points = np.array(points).reshape(-1, 1, 2).astype(int)
    #         gt_mask = cv2.fillPoly(
    #             gt_mask, [points], (label2num_dict[label], 0, 0))
    for i in shapes:
        label = i['label']
        points = i['points']
        if label == 'retainingwall':
            points = np.array(points).reshape(-1, 1, 2).astype(int)
            gt_mask = cv2.fillPoly(
                gt_mask, [points], (label2num_dict[label], 0, 0))
    # for i in shapes_raw:
    #     label = i['label']
    #     points = i['points']
    #     if label == 'retainingwall':
    #         points = np.array(points).reshape(-1, 1, 2).astype(int)
    #         gt_mask = cv2.fillPoly(
    #             gt_mask, [points], (label2num_dict[label], 0, 0))
    # for i in shapes_raw:
    #     label = i['label']
    #     points = i['points']
    #     if label not in ['Backgroundwall', 'retainingwall', 'Road', 'sky', 'rut']:
    #         points = np.array(points).reshape(-1, 1, 2).astype(int)
    #         gt_mask = cv2.fillPoly(
    #             gt_mask, [points], (label2num_dict[label], 0, 0))
    for i in shapes:
        label = i['label']
        points = i['points']
        if label not in ['backgroundwall', 'retainingwall', 'sky', 'road']:
            points = np.array(points).reshape(-1, 1, 2).astype(int)
            gt_mask = cv2.fillPoly(
                gt_mask, [points], (label2num_dict[label], 0, 0))
    for i in shapes:
        label = i['label']
        points = i['points']
        if label == 'puddle':
            points = np.array(points).reshape(-1, 1, 2).astype(int)
            # if cv2.contourArea(points) > 15000:

            gt_mask = cv2.fillPoly(
                gt_mask, [points], (label2num_dict[label], 0, 0))
    return gt_mask


def convertLabelJson2RGBImage(json_path, label2num_dict):
    pass


def convertLabelIdImage2Json(json_path, label2num_dict):
    pass



#!/usr/bin/env python
# coding=utf-8
'''
brief        :  
Author       : knightdby knightdby@163.com
Date         : 2023-02-24 21:54:33
FilePath     : /manifast/manifast/iostream/pltstream.py
Description  : 
LastEditTime : 2023-04-09 12:32:18
LastEditors  : knightdby
Copyright (c) 2023 by Inc, All Rights Reserved.
'''
import numpy as np
import matplotlib.pyplot as plt


def plt_distribut_hist(data_list, x_names, save_name, save_dir='./', x_label='Classes', y_label='Number of Anchors'):
    '''
    description: 绘制数据分布直方图
    param       {*} data_list 数据list(非频数)
    param       {*} x_names x 轴刻度值
    param       {*} save_name 直方图保存名称
    param       {*} x_label x 轴显示名称
    param       {*} y_label y 轴显示名称
    param       {*} save_dir 直方图保存路径
    return      {*} null
    '''

    rects = plt.hist(data_list, bins=np.linspace(
        0, len(set(data_list)), len(set(data_list)) + 1) - 0.5, rwidth=0.8)

    for idx in range(len(rects[0])):
        height = rects[0][idx]
        plt.text(rects[1][idx]+0.1, height*1.01, '%s' % int(height))

    plt.ylabel(y_label)
    plt.xticks(range(len(x_names)), x_names, rotation=45)
    plt.xlabel(x_label)
    plt.tight_layout(pad=0.5)
    plt.savefig(save_dir+save_name+'.png',
                dpi=300, bbox_inches='tight')
    plt.show()


def plt_command():
    # 不显示 xy 轴刻度值
    plt.tick_params(labelbottom=False, labelleft=False)
    # 减小 plt 中多幅 img 之间间距
    plt.tight_layout(pad=0.5)
    plt.savefig('./img.jpg',
                dpi=300, bbox_inches='tight')
